@@ -289,25 +289,31 @@ class SynMem:
                         cursor = conn.cursor()
                         cursor.execute("PRAGMA table_info(memory)")
                         cols = [col[1] for col in cursor.fetchall()]
-                    if 'user' in cols and user:
-                        query_str = "SELECT dtStamp, content, response FROM memory WHERE user = ? COLLATE NOCASE"
-                        params = [user]
+                    # Always select all 4 fields. If the DB doesn't have user, pad it below.
+                    if 'user' in cols:
+                        if user:
+                            query_str = "SELECT user, dtStamp, content, response FROM memory WHERE user = ? COLLATE NOCASE"
+                            params = [user]
+                        else:
+                            query_str = "SELECT user, dtStamp, content, response FROM memory"
+                            params = []
                     else:
-                        query_str = "SELECT dtStamp, content, response FROM memory"
+                        query_str = "SELECT '', dtStamp, content, response FROM memory"
                         params = []
                     db_rows = self.retrieveMemoryFromDb(dbFile, query_str, params)
                     for row in db_rows:
-                        # row[0]=dtStamp, row[1]=content, row[2]=response
-                        rows.append((row[0], row[1], row[2] if len(row) > 2 else ""))
+                        if len(row) == 4:
+                            rows.append(row)
+                        elif len(row) == 3:
+                            rows.append(('', *row))
                 except Exception as ex:
-                    #print(f"Error reading for semantic search: {dbFile}: {ex}")
                     logger.error(f"Error reading for semantic search: {dbFile}: {ex}", exc_info=True)
                     continue
 
         if not rows:
             return []
 
-        contents = [content for (_, content, _) in rows]
+        contents = [content for (_, _, content, _) in rows]
         entry_embeddings = model.encode(contents, show_progress_bar=showProgress)
         query_embedding = model.encode([query], show_progress_bar=showProgress)[0]
 
@@ -315,14 +321,10 @@ class SynMem:
             np.linalg.norm(entry_embeddings, axis=1) * np.linalg.norm(query_embedding)
         )
         top_indices = np.argsort(scores)[::-1][:topK]
-        results = [(rows[i][0], rows[i][1], rows[i][2]) for i in top_indices]
+        results = [rows[i] for i in top_indices]
         return results
 
     def recallRapidMemory(self, query: str, paths: list, user: str = None, topK=5, minScore: int = 60):
-        """
-        Fuzzy string matching version of recallMemory, using Rapidfuzz.
-        Returns topK best-matching (dtStamp, content, response) entries.
-        """
         rows = []
         for path in paths:
             dbFiles = []
@@ -339,34 +341,139 @@ class SynMem:
                         cursor = conn.cursor()
                         cursor.execute("PRAGMA table_info(memory)")
                         cols = [col[1] for col in cursor.fetchall()]
-                    if 'user' in cols and user:
-                        query_str = "SELECT dtStamp, content, response FROM memory WHERE user = ? COLLATE NOCASE"
-                        params = [user]
+                    if 'user' in cols:
+                        if user:
+                            query_str = "SELECT user, dtStamp, content, response FROM memory WHERE user = ? COLLATE NOCASE"
+                            params = [user]
+                        else:
+                            query_str = "SELECT user, dtStamp, content, response FROM memory"
+                            params = []
                     else:
-                        query_str = "SELECT dtStamp, content, response FROM memory"
+                        query_str = "SELECT '', dtStamp, content, response FROM memory"
                         params = []
                     db_rows = self.retrieveMemoryFromDb(dbFile, query_str, params)
                     for row in db_rows:
-                        rows.append((row[0], row[1], row[2] if len(row) > 2 else ""))
+                        if len(row) == 4:
+                            rows.append(row)
+                        elif len(row) == 3:
+                            rows.append(('', *row))
                 except Exception as ex:
-                    #print(f"Error reading for fuzzy search: {dbFile}: {ex}")
                     logger.error(f"Error reading for fuzzy search: {dbFile}: {ex}", exc_info=True)
                     continue
 
         if not rows:
             return []
 
-        # Fuzzy match: use Rapidfuzz's process.extract, returns [(match, score, idx), ...]
-        # We'll just score vs content field
-        contents = [content for (_, content, _) in rows]
+        contents = [content for (_, _, content, _) in rows]
         resultsWithScores = process.extract(query, contents, scorer=fuzz.token_sort_ratio, limit=topK)
-        # resultsWithScores = [(content, score, index), ...]
         results = [
-            (rows[idx][0], rows[idx][1], rows[idx][2])
+            rows[idx]
             for (content, score, idx) in resultsWithScores if score >= minScore
         ]
         return results
 
+
+
+    # def recallEmbeddedMemory(self, query: str, paths: list, user: str = None, topK=5, showProgress: bool = False, minScore: int = 60):
+    #     model = self.embeddingModel
+    #     if model is None:
+    #         logger.error("Embedding model not set. Falling back to Rapid recall. Use synmem.setEmbeddingModel(model).", exc_info=True)
+    #         return self.recallRapidMemory(query, paths, user, topK, minScore)
+
+    #     rows = []
+    #     for path in paths:
+    #         dbFiles = []
+    #         if os.path.isdir(path):
+    #             dbFiles = [os.path.join(path, f) for f in os.listdir(path) if f.endswith('.db')]
+    #         elif os.path.isfile(path):
+    #             dbFiles = [path]
+    #         else:
+    #             continue
+
+    #         for dbFile in dbFiles:
+    #             try:
+    #                 with sqlite3.connect(dbFile) as conn:
+    #                     cursor = conn.cursor()
+    #                     cursor.execute("PRAGMA table_info(memory)")
+    #                     cols = [col[1] for col in cursor.fetchall()]
+    #                 if 'user' in cols and user:
+    #                     query_str = "SELECT dtStamp, content, response FROM memory WHERE user = ? COLLATE NOCASE"
+    #                     params = [user]
+    #                 else:
+    #                     query_str = "SELECT dtStamp, content, response FROM memory"
+    #                     params = []
+    #                 db_rows = self.retrieveMemoryFromDb(dbFile, query_str, params)
+    #                 for row in db_rows:
+    #                     # row[0]=dtStamp, row[1]=content, row[2]=response
+    #                     rows.append((row[0], row[1], row[2] if len(row) > 2 else ""))
+    #             except Exception as ex:
+    #                 #print(f"Error reading for semantic search: {dbFile}: {ex}")
+    #                 logger.error(f"Error reading for semantic search: {dbFile}: {ex}", exc_info=True)
+    #                 continue
+
+    #     if not rows:
+    #         return []
+
+    #     contents = [content for (_, content, _) in rows]
+    #     entry_embeddings = model.encode(contents, show_progress_bar=showProgress)
+    #     query_embedding = model.encode([query], show_progress_bar=showProgress)[0]
+
+    #     scores = np.dot(entry_embeddings, query_embedding) / (
+    #         np.linalg.norm(entry_embeddings, axis=1) * np.linalg.norm(query_embedding)
+    #     )
+    #     top_indices = np.argsort(scores)[::-1][:topK]
+    #     results = [(rows[i][0], rows[i][1], rows[i][2]) for i in top_indices]
+    #     return results
+
+    # def recallRapidMemory(self, query: str, paths: list, user: str = None, topK=5, minScore: int = 60):
+    #     """
+    #     Fuzzy string matching version of recallMemory, using Rapidfuzz.
+    #     Returns topK best-matching (dtStamp, content, response) entries.
+    #     """
+    #     rows = []
+    #     for path in paths:
+    #         dbFiles = []
+    #         if os.path.isdir(path):
+    #             dbFiles = [os.path.join(path, f) for f in os.listdir(path) if f.endswith('.db')]
+    #         elif os.path.isfile(path):
+    #             dbFiles = [path]
+    #         else:
+    #             continue
+
+    #         for dbFile in dbFiles:
+    #             try:
+    #                 with sqlite3.connect(dbFile) as conn:
+    #                     cursor = conn.cursor()
+    #                     cursor.execute("PRAGMA table_info(memory)")
+    #                     cols = [col[1] for col in cursor.fetchall()]
+    #                 if 'user' in cols and user:
+    #                     query_str = "SELECT dtStamp, content, response FROM memory WHERE user = ? COLLATE NOCASE"
+    #                     params = [user]
+    #                 else:
+    #                     query_str = "SELECT dtStamp, content, response FROM memory"
+    #                     params = []
+    #                 db_rows = self.retrieveMemoryFromDb(dbFile, query_str, params)
+    #                 for row in db_rows:
+    #                     rows.append((row[0], row[1], row[2] if len(row) > 2 else ""))
+    #             except Exception as ex:
+    #                 #print(f"Error reading for fuzzy search: {dbFile}: {ex}")
+    #                 logger.error(f"Error reading for fuzzy search: {dbFile}: {ex}", exc_info=True)
+    #                 continue
+
+    #     if not rows:
+    #         return []
+
+    #     # Fuzzy match: use Rapidfuzz's process.extract, returns [(match, score, idx), ...]
+    #     # We'll just score vs content field
+    #     contents = [content for (_, content, _) in rows]
+    #     resultsWithScores = process.extract(query, contents, scorer=fuzz.token_sort_ratio, limit=topK)
+    #     # resultsWithScores = [(content, score, index), ...]
+    #     results = [
+    #         (rows[idx][0], rows[idx][1], rows[idx][2])
+    #         for (content, score, idx) in resultsWithScores if score >= minScore
+    #     ]
+    #     return results
+
     def retrievePerception(self):
         if self.perception:
             perception = "\n".join(self.perception)
@@ -386,22 +493,51 @@ class SynMem:
                         for row in rows:
                             if isinstance(row, tuple) and len(row) > 0:
                                 if returnTimestampOnly:
-                                    memory.append(row[0])
+                                    memory.append(row[1] if len(row) == 4 else row[0])  # Timestamp is always 2nd in 4-tuple or 1st in 3-tuple
                                 else:
-                                    timestamp = row[0]
-                                    rawContent = row[1]
-                                    response = row[2] if len(row) > 2 else ""
-                                    decodedContent = rawContent if isinstance(rawContent, str) else str(rawContent)
-                                    memory.append((timestamp, decodedContent, response))
-                            else:
-                                logger.warning(f"Skipping invalid row format in {dbFile}: {row}")
-                    return memory
+                                    # Always return (user, dtStamp, content, response)
+                                    if len(row) == 4:
+                                        memory.append(row)
+                                    elif len(row) == 3:
+                                        memory.append(('', *row))
+                                    else:
+                                        logger.warning(f"Skipping invalid row format in {dbFile}: {row}")
+                        return memory
             except sqlite3.OperationalError:
                 logger.error(f"OperationalError while retrieving data from {dbFile}:", exc_info=True)
             except Exception:
                 logger.error(f"Unexpected error while retrieving data from {dbFile}:", exc_info=True)
         return memory
 
+
+    # def retrieveMemoryFromDb(self, dbFile, query, params, returnTimestampOnly=False):
+    #     memory = []
+    #     if os.path.exists(dbFile):
+    #         try:
+    #             with self.dbLock, sqlite3.connect(dbFile) as conn:
+    #                 cursor = conn.cursor()
+    #                 cursor.execute(query, params)
+    #                 rows = cursor.fetchall()
+    #                 if rows:
+    #                     for row in rows:
+    #                         if isinstance(row, tuple) and len(row) > 0:
+    #                             if returnTimestampOnly:
+    #                                 memory.append(row[0])
+    #                             else:
+    #                                 timestamp = row[0]
+    #                                 rawContent = row[1]
+    #                                 response = row[2] if len(row) > 2 else ""
+    #                                 decodedContent = rawContent if isinstance(rawContent, str) else str(rawContent)
+    #                                 memory.append((timestamp, decodedContent, response))
+    #                         else:
+    #                             logger.warning(f"Skipping invalid row format in {dbFile}: {row}")
+    #                 return memory
+    #         except sqlite3.OperationalError:
+    #             logger.error(f"OperationalError while retrieving data from {dbFile}:", exc_info=True)
+    #         except Exception:
+    #             logger.error(f"Unexpected error while retrieving data from {dbFile}:", exc_info=True)
+    #     return memory
+
     def retrieveDetailsFromDb(self, dbFile, query, params):
         memory = []
         if os.path.exists(dbFile):
